@@ -4,6 +4,7 @@
  */
 
 #include <net-snmp/net-snmp-config.h>
+#include <net-snmp/net-snmp-features.h>
 #include <net-snmp/net-snmp-includes.h>
 #include <net-snmp/agent/net-snmp-agent-includes.h>
 #include <net-snmp/agent/agent_callbacks.h>
@@ -11,6 +12,7 @@
 #include "disman/event/mteEvent.h"
 #include "disman/event/mteEventConf.h"
 
+netsnmp_feature_require(iquery)
 
 /** Initializes the mteEventsConf module */
 void
@@ -67,8 +69,8 @@ init_mteEventConf(void)
     /*
      * Find or create the specified event entry
      */
-struct mteEvent *
-_find_mteEvent_entry( char *owner, char *ename )
+static struct mteEvent *
+_find_mteEvent_entry( const char *owner, const char *ename )
 {
     netsnmp_variable_list owner_var, ename_var;
     netsnmp_tdata_row *row;
@@ -95,8 +97,8 @@ _find_mteEvent_entry( char *owner, char *ename )
     return (struct mteEvent *)row->data;
 }
 
-struct mteEvent *
-_find_typed_mteEvent_entry( char *owner, char *ename, int type )
+static struct mteEvent *
+_find_typed_mteEvent_entry( const char *owner, const char *ename, int type )
 {
     struct mteEvent *entry = _find_mteEvent_entry( owner, ename );
     if (!entry)
@@ -126,7 +128,6 @@ _find_typed_mteEvent_entry( char *owner, char *ename, int type )
 void
 parse_notificationEvent( const char *token, char *line )
 {
-    char  *owner = "snmpd.conf";
     char   ename[MTE_STR1_LEN+1];
     char   buf[SPRINT_MAX_LEN];
     oid    name_buf[MAX_OID_LEN];
@@ -234,7 +235,7 @@ parse_notificationEvent( const char *token, char *line )
      *  If the entry has parsed successfully, then create,
      *     populate and activate the new event entry.
      */
-    entry = _find_typed_mteEvent_entry(owner, ename+2,
+    entry = _find_typed_mteEvent_entry("snmpd.conf", ename+2,
                                        MTE_EVENT_NOTIFICATION);
     if (!entry) {
         mteObjects_removeEntries( "snmpd.conf", ename );
@@ -255,7 +256,6 @@ parse_notificationEvent( const char *token, char *line )
 void
 parse_setEvent( const char *token, char *line )
 {
-    char  *owner = "snmpd.conf";
     char   ename[MTE_STR1_LEN+1];
     char   buf[SPRINT_MAX_LEN];
     oid    name_buf[MAX_OID_LEN];
@@ -296,13 +296,18 @@ parse_setEvent( const char *token, char *line )
     if (cp && *cp == '=') {
         cp = skip_token( cp );   /* skip the '=' assignment character */
     }
+    if (!cp) {
+        config_perror("syntax error: missing set value");
+        return;
+    }
+
     value = strtol( cp, NULL, 0);
 
     /*
      *  If the entry has parsed successfully, then create,
      *     populate and activate the new event entry.
      */
-    entry = _find_typed_mteEvent_entry(owner, ename, MTE_EVENT_SET);
+    entry = _find_typed_mteEvent_entry("snmpd.conf", ename, MTE_EVENT_SET);
     if (!entry) {
         return;
     }
@@ -466,7 +471,7 @@ int
 store_mteETable(int majorID, int minorID, void *serverarg, void *clientarg)
 {
     char            line[SNMP_MAXBUF];
-    char           *cptr;
+    char           *cptr, *cp;
     void           *vp;
     size_t          tint;
     netsnmp_tdata_row *row;
@@ -496,12 +501,12 @@ store_mteETable(int majorID, int minorID, void *serverarg, void *clientarg)
         strcat(line, "_mteETable ");
         cptr = line + strlen(line);
 
-        vp   = entry->mteOwner;        tint = strlen( vp );
-        cptr = read_config_store_data( ASN_OCTET_STR, cptr, &vp,  &tint );
-        vp   = entry->mteEName;        tint = strlen( vp );
-        cptr = read_config_store_data( ASN_OCTET_STR, cptr, &vp,  &tint );
-        vp   = entry->mteEventComment; tint = strlen( vp );
-        cptr = read_config_store_data( ASN_OCTET_STR, cptr, &vp,  &tint );
+        cp   = entry->mteOwner;        tint = strlen( cp );
+        cptr = read_config_store_data( ASN_OCTET_STR, cptr, &cp,  &tint );
+        cp   = entry->mteEName;        tint = strlen( cp );
+        cptr = read_config_store_data( ASN_OCTET_STR, cptr, &cp,  &tint );
+        cp   = entry->mteEventComment; tint = strlen( cp );
+        cptr = read_config_store_data( ASN_OCTET_STR, cptr, &cp,  &tint );
         /* ... (but skip the mteEventAction field)... */
         tint = entry->flags & (MTE_EVENT_FLAG_ENABLED|MTE_EVENT_FLAG_ACTIVE); 
         cptr = read_config_store_data( ASN_UNSIGNED,  cptr, &tint, NULL );
@@ -517,17 +522,17 @@ store_mteETable(int majorID, int minorID, void *serverarg, void *clientarg)
             strcat(line, "_mteENotTable ");
             cptr = line + strlen(line);
     
-            vp = entry->mteOwner;         tint = strlen( vp );
-            cptr = read_config_store_data(ASN_OCTET_STR, cptr, &vp, &tint );
-            vp = entry->mteEName;         tint = strlen( vp );
-            cptr = read_config_store_data(ASN_OCTET_STR, cptr, &vp, &tint );
+            cp = entry->mteOwner;         tint = strlen( cp );
+            cptr = read_config_store_data(ASN_OCTET_STR, cptr, &cp, &tint );
+            cp = entry->mteEName;         tint = strlen( cp );
+            cptr = read_config_store_data(ASN_OCTET_STR, cptr, &cp, &tint );
             vp   = entry->mteNotification;
             cptr = read_config_store_data(ASN_OBJECT_ID, cptr, &vp,
                                           &entry->mteNotification_len);
-            vp = entry->mteNotifyOwner;   tint = strlen( vp );
-            cptr = read_config_store_data(ASN_OCTET_STR, cptr, &vp, &tint );
-            vp = entry->mteNotifyObjects; tint = strlen( vp );
-            cptr = read_config_store_data(ASN_OCTET_STR, cptr, &vp, &tint );
+            cp = entry->mteNotifyOwner;   tint = strlen( cp );
+            cptr = read_config_store_data(ASN_OCTET_STR, cptr, &cp, &tint );
+            cp = entry->mteNotifyObjects; tint = strlen( cp );
+            cptr = read_config_store_data(ASN_OCTET_STR, cptr, &cp, &tint );
             snmpd_store_config(line);
         }
 
@@ -536,19 +541,19 @@ store_mteETable(int majorID, int minorID, void *serverarg, void *clientarg)
             strcat(line, "_mteESetTable ");
             cptr = line + strlen(line);
     
-            vp = entry->mteOwner;         tint = strlen( vp );
-            cptr = read_config_store_data(ASN_OCTET_STR, cptr, &vp, &tint );
-            vp = entry->mteEName;         tint = strlen( vp );
-            cptr = read_config_store_data(ASN_OCTET_STR, cptr, &vp, &tint );
+            cp = entry->mteOwner;         tint = strlen( cp );
+            cptr = read_config_store_data(ASN_OCTET_STR, cptr, &cp, &tint );
+            cp = entry->mteEName;         tint = strlen( cp );
+            cptr = read_config_store_data(ASN_OCTET_STR, cptr, &cp, &tint );
             vp   = entry->mteSetOID;
             cptr = read_config_store_data(ASN_OBJECT_ID, cptr, &vp,
                                           &entry->mteSetOID_len);
             tint = entry->mteSetValue;
             cptr = read_config_store_data(ASN_INTEGER,   cptr, &tint, NULL);
-            vp = entry->mteSetTarget;     tint = strlen( vp );
-            cptr = read_config_store_data(ASN_OCTET_STR, cptr, &vp, &tint );
-            vp = entry->mteSetContext;    tint = strlen( vp );
-            cptr = read_config_store_data(ASN_OCTET_STR, cptr, &vp, &tint );
+            cp = entry->mteSetTarget;     tint = strlen( cp );
+            cptr = read_config_store_data(ASN_OCTET_STR, cptr, &cp, &tint );
+            cp = entry->mteSetContext;    tint = strlen( cp );
+            cptr = read_config_store_data(ASN_OCTET_STR, cptr, &cp, &tint );
             tint = entry->flags & (MTE_SET_FLAG_OBJWILD|MTE_SET_FLAG_CTXWILD); 
             cptr = read_config_store_data(ASN_UNSIGNED,  cptr, &tint, NULL);
             snmpd_store_config(line);
