@@ -4,11 +4,16 @@
  */
 
 #include <net-snmp/net-snmp-config.h>
+#include <net-snmp/net-snmp-features.h>
 #include <net-snmp/net-snmp-includes.h>
 #include <net-snmp/agent/net-snmp-agent-includes.h>
 #include "disman/event/mteTrigger.h"
 #include "disman/event/mteEvent.h"
 
+netsnmp_feature_child_of(disman_debugging, libnetsnmpmibs)
+netsnmp_feature_child_of(mtetrigger, libnetsnmpmibs)
+netsnmp_feature_child_of(mtetrigger_removeentry, mtetrigger)
+
 netsnmp_tdata *trigger_table_data;
 
 oid    _sysUpTime_instance[] = { 1, 3, 6, 1, 2, 1, 1, 3, 0 };
@@ -31,7 +36,7 @@ init_trigger_table_data(void)
             snmp_log(LOG_ERR, "failed to create mteTriggerTable");
             return;
         }
-        DEBUGMSGTL(("disman:event:init", "create trigger container (%x)\n",
+        DEBUGMSGTL(("disman:event:init", "create trigger container (%p)\n",
                                           trigger_table_data));
     }
     mteTriggerFailures = 0;
@@ -53,6 +58,7 @@ init_mteTrigger(void)
      *
      * =================================================== */
 
+#ifndef NETSNMP_FEATURE_REMOVE_DISMAN_DEBUGGING
 void
 _mteTrigger_dump(void)
 {
@@ -69,18 +75,18 @@ _mteTrigger_dump(void)
         DEBUGMSG(("disman:event:dump", "(%s, %s)",
                                          row->indexes->val.string,
                                          row->indexes->next_variable->val.string));
-        DEBUGMSG(("disman:event:dump", ": %x, %x\n", row, entry));
+        DEBUGMSG(("disman:event:dump", ": %p, %p\n", row, entry));
         i++;
     }
     DEBUGMSGTL(("disman:event:dump", "TriggerTable %d entries\n", i));
 }
-
+#endif /* NETSNMP_FEATURE_REMOVE_DISMAN_DEBUGGING */
 
 /*
  * Create a new row in the trigger table 
  */
 netsnmp_tdata_row *
-mteTrigger_createEntry(char *mteOwner, char *mteTName, int fixed)
+mteTrigger_createEntry(const char *mteOwner, char *mteTName, int fixed)
 {
     struct mteTrigger *entry;
     netsnmp_tdata_row *row;
@@ -142,6 +148,7 @@ mteTrigger_createEntry(char *mteOwner, char *mteTName, int fixed)
     return row;
 }
 
+#ifndef NETSNMP_FEATURE_REMOVE_MTETRIGGER_REMOVEENTRY
 /*
  * Remove a row from the trigger table 
  */
@@ -159,6 +166,7 @@ mteTrigger_removeEntry(netsnmp_tdata_row *row)
         SNMP_FREE(entry);
     }
 }
+#endif /* NETSNMP_FEATURE_REMOVE_MTETRIGGER_REMOVEENTRY */
 
     /* ===================================================
      *
@@ -247,6 +255,7 @@ mteTrigger_run( unsigned int reg, void *clientarg)
         DEBUGMSGTL(( "disman:event:trigger:monitor", "Trigger query (%s) failed: %d\n",
                            (( entry->flags & MTE_TRIGGER_FLAG_VWILD ) ? "walk" : "get"), n));
         _mteTrigger_failure( "failed to run mteTrigger query" );
+        snmp_free_varbind(var);
         return;
     }
 
@@ -291,6 +300,7 @@ mteTrigger_run( unsigned int reg, void *clientarg)
                 if (!vtmp) {
                     _mteTrigger_failure(
                           "failed to create mteTrigger temp varbind");
+                    snmp_free_varbind(var);
                     return;
                 }
                 vtmp->type = ASN_NULL;
@@ -328,6 +338,7 @@ mteTrigger_run( unsigned int reg, void *clientarg)
                     if (!vtmp) {
                         _mteTrigger_failure(
                                  "failed to create mteTrigger temp varbind");
+                        snmp_free_varbind(var);
                         return;
                     }
                     vtmp->type = ASN_NULL;
@@ -336,7 +347,7 @@ mteTrigger_run( unsigned int reg, void *clientarg)
                     if (vp1_prev) {
                         vp1_prev->next_variable = vtmp;
                     } else {
-                        entry->old_results      = vtmp;
+                        var                     = vtmp;
                     }
                     vp1_prev = vtmp;
                     vp2_prev = vp2;
@@ -369,6 +380,7 @@ mteTrigger_run( unsigned int reg, void *clientarg)
                 if (!vtmp) {
                     _mteTrigger_failure(
                              "failed to create mteTrigger temp varbind");
+                    snmp_free_varbind(var);
                     return;
                 }
                 vtmp->type = ASN_NULL;
@@ -433,7 +445,7 @@ mteTrigger_run( unsigned int reg, void *clientarg)
              */
             if (entry->mteTExTest & entry->mteTExStartup & MTE_EXIST_ABSENT) {
                 if (!(entry->flags & MTE_TRIGGER_FLAG_VWILD) &&
-                    vp1->type == ASN_NULL ) {
+                    var->type == ASN_NULL ) {
                     DEBUGMSGTL(( "disman:event:trigger:fire",
                                  "Firing initial existence test: "));
                     DEBUGMSGOID(("disman:event:trigger:fire",
@@ -449,10 +461,10 @@ mteTrigger_run( unsigned int reg, void *clientarg)
                      *  to report a NULL value, but this clashes with
                      * the syntax of the mteHotValue MIB object.
                      */
-                    entry->mteTriggerFired    = vp1;
+                    entry->mteTriggerFired    = var;
                     n = entry->mteTriggerValueID_len;
                     mteEvent_fire(entry->mteTExEvOwner, entry->mteTExEvent, 
-                                  entry, vp1->name+n, vp1->name_length-n);
+                                  entry, var->name+n, var->name_length-n);
                 }
             }
         } /* !old_results */
@@ -529,6 +541,13 @@ mteTrigger_run( unsigned int reg, void *clientarg)
         } /* !old_results - end of else block */
     } /* MTE_TRIGGER_EXISTENCE */
 
+    /*
+     * We'll need sysUpTime.0 regardless...
+     */
+    DEBUGMSGTL(("disman:event:delta", "retrieve sysUpTime.0\n"));
+    memset( &sysUT_var, 0, sizeof( netsnmp_variable_list ));
+    snmp_set_var_objid( &sysUT_var, _sysUpTime_instance, _sysUpTime_inst_len );
+    netsnmp_query_get(  &sysUT_var, entry->session );
 
     if (( entry->mteTriggerTest & MTE_TRIGGER_BOOLEAN   ) ||
         ( entry->mteTriggerTest & MTE_TRIGGER_THRESHOLD )) {
@@ -578,14 +597,6 @@ mteTrigger_run( unsigned int reg, void *clientarg)
          * (including sysUpTime.0 if not specified explicitly).
          */
         if ( entry->flags & MTE_TRIGGER_FLAG_DELTA ) {
-            /*
-             * We'll need sysUpTime.0 regardless...
-             */
-            DEBUGMSGTL(("disman:event:delta", "retrieve sysUpTime.0\n"));
-            memset( &sysUT_var, 0, sizeof( netsnmp_variable_list ));
-            snmp_set_var_objid( &sysUT_var, _sysUpTime_instance,
-                                            _sysUpTime_inst_len );
-            netsnmp_query_get(  &sysUT_var, entry->session );
 
             if (!(entry->flags & MTE_TRIGGER_FLAG_SYSUPT)) {
                 /*
@@ -609,9 +620,9 @@ mteTrigger_run( unsigned int reg, void *clientarg)
                 snmp_set_var_objid( dvar, entry->mteDeltaDiscontID,
                                           entry->mteDeltaDiscontID_len );
                 if ( entry->flags & MTE_TRIGGER_FLAG_DWILD ) {
-                    netsnmp_query_walk( dvar, entry->session );
+                    n = netsnmp_query_walk( dvar, entry->session );
                 } else {
-                    netsnmp_query_get(  dvar, entry->session );
+                    n = netsnmp_query_get(  dvar, entry->session );
                 }
                 if ( n != SNMP_ERR_NOERROR ) {
                     _mteTrigger_failure( "failed to run mteTrigger delta query" );
@@ -704,6 +715,7 @@ mteTrigger_run( unsigned int reg, void *clientarg)
                         if (!vtmp) {
                             _mteTrigger_failure(
                                   "failed to create mteTrigger discontinuity varbind");
+                            snmp_free_varbind(dvar);
                             return;
                         }
                         snmp_set_var_objid(vtmp, entry->mteDeltaDiscontID,
@@ -803,7 +815,7 @@ mteTrigger_run( unsigned int reg, void *clientarg)
     DEBUGMSGTL(( "disman:event:delta", "delta sample: "));
     DEBUGMSGOID(("disman:event:delta", vp1->name,
                                        vp1->name_length ));
-    DEBUGMSG((   "disman:event:delta", " (%d - %d) = %d\n",
+    DEBUGMSG((   "disman:event:delta", " (%ld - %ld) = %ld\n",
                 *vp1->val.integer,  *vp2->val.integer, value));
                 vp2 = vp2->next_variable;
             } else {
@@ -833,7 +845,7 @@ mteTrigger_run( unsigned int reg, void *clientarg)
                 cmp = ( value >= entry->mteTBoolValue );
                 break;
             }
-    DEBUGMSGTL(( "disman:event:delta", "Bool comparison: (%d %s %d) %d\n",
+    DEBUGMSGTL(( "disman:event:delta", "Bool comparison: (%ld %s %ld) %d\n",
                           value, _ops[entry->mteTBoolComparison],
                           entry->mteTBoolValue, cmp));
 
@@ -843,16 +855,22 @@ mteTrigger_run( unsigned int reg, void *clientarg)
              *     to remember whether the trigger has already fired)
              */
             if ( cmp ) {
-                if ((entry->old_results ||
-                     (entry->flags & MTE_TRIGGER_FLAG_BSTART)) &&
-                    (vp1->index & MTE_ARMED_BOOLEAN )) {
+              if (vp1->index & MTE_ARMED_BOOLEAN ) {
+                vp1->index &= ~MTE_ARMED_BOOLEAN;
+                /*
+                 * NB: Clear the trigger armed flag even if the
+                 *   (starting) event dosn't actually fire.
+                 *   Otherwise initially true (but suppressed)
+                 *   triggers will fire on the *second* probe.
+                 */
+                if ( entry->old_results ||
+                    (entry->flags & MTE_TRIGGER_FLAG_BSTART)) {
                     DEBUGMSGTL(( "disman:event:trigger:fire",
                                  "Firing boolean test: "));
                     DEBUGMSGOID(("disman:event:trigger:fire",
                                  vp1->name, vp1->name_length));
                     DEBUGMSG((   "disman:event:trigger:fire", "%s\n",
                                   (entry->old_results ? "" : " (startup)")));
-                    vp1->index &= ~MTE_ARMED_BOOLEAN;
                     entry->mteTriggerXOwner   = entry->mteTBoolObjOwner;
                     entry->mteTriggerXObjects = entry->mteTBoolObjects;
                     /*
@@ -865,6 +883,7 @@ mteTrigger_run( unsigned int reg, void *clientarg)
                     mteEvent_fire(entry->mteTBoolEvOwner, entry->mteTBoolEvent, 
                                   entry, vp1->name+n, vp1->name_length-n);
                 }
+              }
             } else {
                 vp1->index |= MTE_ARMED_BOOLEAN;
             }
@@ -940,17 +959,24 @@ mteTrigger_run( unsigned int reg, void *clientarg)
              */
             cmp = vp1->index;   /* working copy of 'armed' flags */
             if ( value >= entry->mteTThRiseValue ) {
-                if ((entry->old_results ||
-                     (entry->mteTThStartup & MTE_THRESH_START_RISE)) &&
-                    (vp1->index & MTE_ARMED_TH_RISE )) {
+              if (cmp & MTE_ARMED_TH_RISE ) {
+                cmp &= ~MTE_ARMED_TH_RISE;
+                cmp |=  MTE_ARMED_TH_FALL;
+                /*
+                 * NB: Clear the trigger armed flag even if the
+                 *   (starting) event dosn't actually fire.
+                 *   Otherwise initially true (but suppressed)
+                 *   triggers will fire on the *second* probe.
+                 * Similarly for falling thresholds (see below).
+                 */
+                if ( entry->old_results ||
+                    (entry->mteTThStartup & MTE_THRESH_START_RISE)) {
                     DEBUGMSGTL(( "disman:event:trigger:fire",
                                  "Firing rising threshold test: "));
                     DEBUGMSGOID(("disman:event:trigger:fire",
                                  vp1->name, vp1->name_length));
                     DEBUGMSG((   "disman:event:trigger:fire", "%s\n",
                                  (entry->old_results ? "" : " (startup)")));
-                    cmp &= ~MTE_ARMED_TH_RISE;
-                    cmp |=  MTE_ARMED_TH_FALL;
                     /*
                      * If no riseEvent is configured, we need still to
                      *  set the armed flags appropriately, but there's
@@ -966,20 +992,22 @@ mteTrigger_run( unsigned int reg, void *clientarg)
                                       entry, vp1->name+n, vp1->name_length-n);
                     }
                 }
+              }
             }
 
             if ( value <= entry->mteTThFallValue ) {
-                if ((entry->old_results ||
-                     (entry->mteTThStartup & MTE_THRESH_START_FALL)) &&
-                    (vp1->index & MTE_ARMED_TH_FALL )) {
+              if (cmp & MTE_ARMED_TH_FALL ) {
+                cmp &= ~MTE_ARMED_TH_FALL;
+                cmp |=  MTE_ARMED_TH_RISE;
+                /* Clear the trigger armed flag (see above) */
+                if ( entry->old_results ||
+                    (entry->mteTThStartup & MTE_THRESH_START_FALL)) {
                     DEBUGMSGTL(( "disman:event:trigger:fire",
                                  "Firing falling threshold test: "));
                     DEBUGMSGOID(("disman:event:trigger:fire",
                                  vp1->name, vp1->name_length));
                     DEBUGMSG((   "disman:event:trigger:fire", "%s\n",
                                  (entry->old_results ? "" : " (startup)")));
-                    cmp &= ~MTE_ARMED_TH_FALL;
-                    cmp |=  MTE_ARMED_TH_RISE;
                     /*
                      * Similarly, if no fallEvent is configured,
                      *  there's no point in trying to fire it either.
@@ -994,6 +1022,7 @@ mteTrigger_run( unsigned int reg, void *clientarg)
                                       entry, vp1->name+n, vp1->name_length-n);
                     }
                 }
+              }
             }
             vp1->index = cmp;
         }
