@@ -3,9 +3,10 @@
  *
  *  Based on patch 1362403, submited by Rojer
  *
- * $Id: interface_sysctl.c 19822 2011-01-07 16:48:56Z dts12 $
+ * $Id$
  */
 #include <net-snmp/net-snmp-config.h>
+#include <net-snmp/net-snmp-features.h>
 #include <net-snmp/net-snmp-includes.h>
 #include "mibII/mibII_common.h"
 #include "if-mib/ifTable/ifTable_constants.h"
@@ -29,6 +30,8 @@
 #include <net/if_types.h>
 #include <net/if_media.h>
 
+netsnmp_feature_child_of(interface_arch_set_admin_status, interface_all)
+
 /*
  * account for minor differences between FreeBSD and OpenBSD.
  * If this gets unruly, split stuff out into the respective
@@ -49,7 +52,7 @@
 #   define ARCH_PROMISC_FLAG IFF_PROMISC
 #endif
 
-extern struct timeval starttime;
+#define starttime (*(const struct timeval*)netsnmp_get_agent_starttime())
 
 /* sa_len roundup macro. */
 #define ROUNDUP(a) \
@@ -249,8 +252,8 @@ netsnmp_sysctl_get_if_speed(char *name, u_int *speed,
         return 0;
     }
 
-    (void) memset(&ifmr, 0, sizeof(ifmr));
-    (void) strncpy(ifmr.ifm_name, name, sizeof(ifmr.ifm_name));
+    memset(&ifmr, 0, sizeof(ifmr));
+    strlcpy(ifmr.ifm_name, name, sizeof(ifmr.ifm_name));
 
     DEBUGMSGTL(("access:interface:container:sysctl"," speed in\n"));
     if (ioctl(s, SIOCGIFMEDIA, (caddr_t)&ifmr) < 0 ||
@@ -281,7 +284,7 @@ netsnmp_sysctl_get_if_speed(char *name, u_int *speed,
                 netsnmp_sysctl_ifmedia_to_speed(media_list[i], &t_speed,
                                                 &t_speed_high);
                 if (t_speed_high > m_speed_high ||
-                    (t_speed_high == m_speed_high && t_speed > t_speed)) {
+                    (t_speed_high == m_speed_high && t_speed > m_speed)) {
                     m_speed_high = t_speed_high;
                     m_speed = t_speed;
                 }
@@ -301,6 +304,12 @@ netsnmp_sysctl_get_if_speed(char *name, u_int *speed,
     return *speed;
 }
 
+static void set_counter(struct counter64 *c, uint64_t v)
+{
+    c->low = (uint32_t)(v);
+    c->high = (v) >> 32;
+}
+
 /*
  *
  * @retval  0 success
@@ -323,9 +332,14 @@ netsnmp_arch_interface_container_load(netsnmp_container* container,
     int amask;
     char *if_name;
     int flags;
+#ifdef HAVE_STRUCT_IFNET_IF_LASTCHANGE_TV_NSEC
+    struct timespec startspec;
+
+    TIMEVAL_TO_TIMESPEC(&starttime, &startspec);
+#endif
 
     DEBUGMSGTL(("access:interface:container:sysctl",
-                "load (flags %p)\n", load_flags));
+                "load (flags %u)\n", load_flags));
 
     if (NULL == container) {
         snmp_log(LOG_ERR, "no container specified/found for interface\n");
@@ -338,14 +352,15 @@ netsnmp_arch_interface_container_load(netsnmp_container* container,
         return -2;
     }
 
-    if_list = malloc(if_list_size);
+    if_list = (u_char*)malloc(if_list_size);
     if (if_list == NULL) {
         snmp_log(LOG_ERR, "could not allocate memory for interface info "
-                 "(%lu bytes)\n", if_list_size);
+                 "(%u bytes)\n", (unsigned) if_list_size);
         return -3;
     } else {
         DEBUGMSGTL(("access:interface:container:sysctl",
-                    "allocated %lu bytes for if_list\n", if_list_size));
+                    "allocated %u bytes for if_list\n",
+                    (unsigned) if_list_size));
     }
 
     if (sysctl(sysctl_oid, sizeof(sysctl_oid)/sizeof(int), if_list,
@@ -402,7 +417,7 @@ netsnmp_arch_interface_container_load(netsnmp_container* container,
         /* get physical address */
         if (adl != NULL && adl->sdl_alen > 0) {
             entry->paddr_len = adl->sdl_alen;
-            entry->paddr = malloc(entry->paddr_len);
+            entry->paddr = (char*)malloc(entry->paddr_len);
             memcpy(entry->paddr, adl->sdl_data + adl->sdl_nlen, adl->sdl_alen);
             DEBUGMSGTL(("access:interface:container:sysctl",
                         "%s: paddr_len=%d, entry->paddr=%x:%x:%x:%x:%x:%x\n",
@@ -410,7 +425,7 @@ netsnmp_arch_interface_container_load(netsnmp_container* container,
                         entry->paddr[0], entry->paddr[1], entry->paddr[2],
                         entry->paddr[3], entry->paddr[4], entry->paddr[5]));
         } else {
-            entry->paddr = malloc(6);
+            entry->paddr = (char*)malloc(6);
             entry->paddr_len = 6;
             memset(entry->paddr, 0, 6);
         }
@@ -453,28 +468,26 @@ netsnmp_arch_interface_container_load(netsnmp_container* container,
             NETSNMP_INTERFACE_FLAGS_HAS_V6_REASMMAX;
 
         /* get counters */
-        entry->stats.ibytes.low  = ifp->ifm_data.ifi_ibytes   & 0xffffffff;
-        entry->stats.ibytes.high = ifp->ifm_data.ifi_ibytes   >> 32;
-        entry->stats.iucast.low  = ifp->ifm_data.ifi_ipackets & 0xffffffff;
-        entry->stats.iucast.high = ifp->ifm_data.ifi_ipackets >> 32;
-        entry->stats.imcast.low  = ifp->ifm_data.ifi_imcasts  & 0xffffffff;
-        entry->stats.imcast.high = ifp->ifm_data.ifi_imcasts  >> 32;
+        set_counter(&entry->stats.ibytes, ifp->ifm_data.ifi_ibytes);
+        set_counter(&entry->stats.iucast, ifp->ifm_data.ifi_ipackets);
+        set_counter(&entry->stats.imcast, ifp->ifm_data.ifi_imcasts);
         entry->stats.ierrors = ifp->ifm_data.ifi_ierrors;
         entry->stats.idiscards = ifp->ifm_data.ifi_iqdrops;
         entry->stats.iunknown_protos = ifp->ifm_data.ifi_noproto;
 
-        entry->stats.obytes.low  = ifp->ifm_data.ifi_obytes   & 0xffffffff;
-        entry->stats.obytes.high = ifp->ifm_data.ifi_obytes   >> 32;
-        entry->stats.oucast.low  = ifp->ifm_data.ifi_opackets & 0xffffffff;
-        entry->stats.oucast.high = ifp->ifm_data.ifi_opackets >> 32;
-        entry->stats.omcast.low  = ifp->ifm_data.ifi_omcasts  & 0xffffffff;
-        entry->stats.omcast.high = ifp->ifm_data.ifi_omcasts  >> 32;
+        set_counter(&entry->stats.obytes, ifp->ifm_data.ifi_obytes);
+        set_counter(&entry->stats.oucast, ifp->ifm_data.ifi_opackets);
+        set_counter(&entry->stats.omcast, ifp->ifm_data.ifi_omcasts);
         entry->stats.oerrors = ifp->ifm_data.ifi_oerrors;
         entry->ns_flags |=  NETSNMP_INTERFACE_FLAGS_HAS_BYTES |
                             NETSNMP_INTERFACE_FLAGS_HAS_DROPS |
                             NETSNMP_INTERFACE_FLAGS_HAS_MCAST_PKTS;
 
+#ifdef HAVE_STRUCT_IFNET_IF_LASTCHANGE_TV_NSEC
+        if (timespeccmp(&ifp->ifm_data.ifi_lastchange, &startspec, >)) {
+#else
         if (timercmp(&ifp->ifm_data.ifi_lastchange, &starttime, >)) {
+#endif
             entry->lastchange = (ifp->ifm_data.ifi_lastchange.tv_sec -
                                  starttime.tv_sec) * 100;
             entry->ns_flags |= NETSNMP_INTERFACE_FLAGS_HAS_LASTCHANGE;
@@ -499,7 +512,7 @@ netsnmp_arch_interface_container_load(netsnmp_container* container,
 
         CONTAINER_INSERT(container, entry);
         DEBUGMSGTL(("access:interface:container:sysctl",
-                    "created entry %u for %s\n", entry->index, entry->name));
+                    "created entry %d for %s\n", (int)entry->index, entry->name));
     } /* for (each interface entry) */
 
     /* pass 2: walk addresses */
@@ -551,6 +564,7 @@ netsnmp_arch_interface_container_load(netsnmp_container* container,
     return 0;
 }
 
+#ifndef NETSNMP_FEATURE_REMOVE_INTERFACE_ARCH_SET_ADMIN_STATUS
 int
 netsnmp_arch_set_admin_status(netsnmp_interface_entry * entry,
                               int ifAdminStatus_val)
@@ -565,3 +579,4 @@ netsnmp_arch_set_admin_status(netsnmp_interface_entry * entry,
 
     return -4;
 }
+#endif /* NETSNMP_FEATURE_REMOVE_INTERFACE_ARCH_SET_ADMIN_STATUS */
