@@ -1,4 +1,5 @@
 #include <net-snmp/net-snmp-config.h>
+#include <net-snmp/net-snmp-features.h>
 
 #include <sys/types.h>
 #if HAVE_NETINET_IN_H
@@ -16,9 +17,15 @@
 #include <net-snmp/agent/table.h>
 #include <net-snmp/agent/table_data.h>
 #include <net-snmp/agent/table_dataset.h>
-#include "mibgroup/mibII/sysORTable.h"
+#include "net-snmp/agent/sysORTable.h"
 #include "notification_log.h"
 
+netsnmp_feature_require(register_ulong_instance_context)
+netsnmp_feature_require(register_read_only_counter32_instance_context)
+netsnmp_feature_require(delete_table_data_set)
+netsnmp_feature_require(table_dataset)
+netsnmp_feature_require(date_n_time)
+
 /*
  * column number definitions for table nlmLogTable
  */
@@ -129,11 +136,9 @@ check_log_size(unsigned int clientreg, void *clientarg)
     netsnmp_table_row *row;
     netsnmp_table_data_set_storage *data;
     u_long          count = 0;
-    struct timeval  now;
     u_long          uptime;
 
-    gettimeofday(&now, NULL);
-    uptime = netsnmp_timeval_uptime(&now);
+    uptime = netsnmp_get_agent_uptime();
 
     if (!nlmLogTable || !nlmLogTable->table )  {
         DEBUGMSGTL(("notification_log", "missing log table\n"));
@@ -145,11 +150,11 @@ check_log_size(unsigned int clientreg, void *clientarg)
      */
     count = netsnmp_table_set_num_rows(nlmLogTable);
     DEBUGMSGTL(("notification_log",
-                "logged notifications %d; max %d\n",
+                "logged notifications %lu; max %lu\n",
                     count, max_logged));
     if (count > max_logged) {
         count = count - max_logged;
-        DEBUGMSGTL(("notification_log", "removing %d extra notifications\n",
+        DEBUGMSGTL(("notification_log", "removing %lu extra notifications\n",
                     count));
         netsnmp_notif_log_remove_oldest(count);
     }
@@ -167,13 +172,13 @@ check_log_size(unsigned int clientreg, void *clientarg)
         data = (netsnmp_table_data_set_storage *) row->data;
         data = netsnmp_table_data_set_find_column(data, COLUMN_NLMLOGTIME);
 
-        if (uptime < ((long)(*(data->data.integer) + max_age * 100 * 60)))
+        if (uptime < ((u_long)(*(data->data.integer) + max_age * 100 * 60)))
             break;
         ++count;
     }
 
     if (count) {
-        DEBUGMSGTL(("notification_log", "removing %d expired notifications\n",
+        DEBUGMSGTL(("notification_log", "removing %lu expired notifications\n",
                     count));
         netsnmp_notif_log_remove_oldest(count);
     }
@@ -469,8 +474,10 @@ notification_log_config_handler(netsnmp_mib_handler *handler,
      * configuration variables get set to a value and thus
      * notifications must be possibly deleted from our archives.
      */
+#ifndef NETSNMP_NO_WRITE_SUPPORT
     if (reqinfo->mode == MODE_SET_COMMIT)
         check_log_size(0, NULL);
+#endif /* !NETSNMP_NO_WRITE_SUPPORT */
     return SNMP_ERR_NOERROR;
 }
 
@@ -564,14 +571,13 @@ shutdown_notification_log(void)
     netsnmp_delete_table_data_set(nlmLogTable);
     nlmLogTable = NULL;
 
-    unregister_sysORTable(nlm_module_oid, OID_LENGTH(nlm_module_oid));
+    UNREGISTER_SYSOR_ENTRY(nlm_module_oid);
 }
 
 void
 log_notification(netsnmp_pdu *pdu, netsnmp_transport *transport)
 {
     long            tmpl;
-    struct timeval  now;
     netsnmp_table_row *row;
 
     static u_long   default_num = 0;
@@ -611,10 +617,9 @@ log_notification(netsnmp_pdu *pdu, netsnmp_transport *transport)
     /*
      * add the data 
      */
-    gettimeofday(&now, NULL);
-    tmpl = netsnmp_timeval_uptime(&now);
+    tmpl = netsnmp_get_agent_uptime();
     netsnmp_set_row_column(row, COLUMN_NLMLOGTIME, ASN_TIMETICKS,
-                           (u_char *) & tmpl, sizeof(tmpl));
+                           &tmpl, sizeof(tmpl));
     time(&timetnow);
     logdate = date_n_time(&timetnow, &logdate_size);
     netsnmp_set_row_column(row, COLUMN_NLMLOGDATEANDTIME, ASN_OCTET_STR,
@@ -644,9 +649,9 @@ log_notification(netsnmp_pdu *pdu, netsnmp_transport *transport)
     }
     if (transport)
         netsnmp_set_row_column(row, COLUMN_NLMLOGENGINETDOMAIN,
-                                     ASN_OBJECT_ID,
-                                     (const u_char *) transport->domain,
-                                     sizeof(oid) * transport->domain_length);
+                               ASN_OBJECT_ID,
+                               transport->domain,
+                               sizeof(oid) * transport->domain_length);
     netsnmp_set_row_column(row, COLUMN_NLMLOGCONTEXTENGINEID,
                            ASN_OCTET_STR, pdu->contextEngineID,
                            pdu->contextEngineIDLen);
@@ -680,7 +685,7 @@ log_notification(netsnmp_pdu *pdu, netsnmp_transport *transport)
              * OID 
              */
             netsnmp_set_row_column(myrow, COLUMN_NLMLOGVARIABLEID,
-                                   ASN_OBJECT_ID, (u_char *) vptr->name,
+                                   ASN_OBJECT_ID, vptr->name,
                                    vptr->name_length * sizeof(oid));
 
             /*
@@ -742,7 +747,7 @@ log_notification(netsnmp_pdu *pdu, netsnmp_transport *transport)
                 continue;
             }
             netsnmp_set_row_column(myrow, COLUMN_NLMLOGVARIABLEVALUETYPE,
-                                   ASN_INTEGER, (u_char *) & tmpul,
+                                   ASN_INTEGER, & tmpul,
                                    sizeof(tmpul));
             netsnmp_set_row_column(myrow, col, vptr->type,
                                    vptr->val.string, vptr->val_len);
